using Abp.Application.Services;
using Abp.Application.Services.Dto;
using System;
using System.Collections.Generic;
using System.Text;
using System.Threading.Tasks;
using YGNT.QuestionLibrary.Grades.Dto;

namespace YGNT.QuestionLibrary.Grades
{
    /// <summary>
    /// 年级管理
    /// </summary>
    public interface IGradesAppService : IApplicationService
    {
        /// <summary>
        /// 查询年级列表
        /// </summary>
        /// <param name="search"></param>
        /// <returns></returns>
        Task<PagedResultDto<GradeDto>> GetListAsync(SearchGradeInputDto search);

        /// <summary>
        /// 查询年级详情
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        Task<GradeDto> GetOneAsync(int id);


        /// <summary>
        /// 创建新年级
        /// </summary>
        /// <param name="create"></param>
        /// <returns></returns>
        Task<int> CreateAsync(CreateGradeDto create);


        /// <summary>
        /// 修改年级
        /// </summary>
        /// <param name="id"></param>
        /// <param name="update"></param>
        /// <returns></returns>
        Task UpdateAsync(int id, UpdateGradeDto update);

        /// <summary>
        /// 删除年级
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        Task DeleteAsync(int id);

        /// <summary>
        /// 删除年级
        /// </summary>
        /// <param name="idList"></param>
        /// <returns></returns>
        Task DeleteListAsync(List<int> idList);
        /// <summary>
        /// 同步年级
        /// </summary>
        /// <returns></returns>
        Task DataSyncAsync();
    }
}
